package com.sillycycle.bagleyd.util;

/*
 * @(#)Spinner.java
 *
 * Mostly from Graphic Java Mastering the JFC Volume 1: AWT
 * 3rd Edition by David M. Geary, Example 15-2 pp 544-545
 */

import java.awt.BorderLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeListener;

/**
 * The <code>Spinner</code> class combines a JSpinner and a Label
 * in one widget.
 *
 * @author David M. Geary
 */

public class Spinner extends JPanel {
	private static final long serialVersionUID = 42L;
	String string;
	JSpinner spinner;
	SpinnerNumberModel model;
	JLabel label;
	int inc;
	static final int DEFAULT_INC = 1;

	public Spinner(String labelString, int initialValue,
			int min, int max, int incrementValue) {
		inc = incrementValue;
		string = labelString + ": ";
		label = new JLabel(string);
		model = new SpinnerNumberModel(initialValue, min, max, inc);
		spinner = new JSpinner(model);
		setLayout(new BorderLayout());
		add(label, BorderLayout.LINE_START);
		add(spinner, BorderLayout.CENTER);

		spinner.addChangeListener(evt -> {
			// stateChanged method that is unused by this class
		});
	}

	public Spinner(String labelString, int initialValue,
			int min, int max) {
		this(labelString, initialValue, min, max, DEFAULT_INC);
	}

	public Spinner(String labelString, int initialValue, int min) {
		this(labelString, initialValue, min, 99, DEFAULT_INC);
	}

	public void addChangeListener(ChangeListener l) {
		spinner.addChangeListener(l);
	}

	public void removeChangeListener(ChangeListener l) {
		spinner.removeChangeListener(l);
	}

	public synchronized int getValue() {
		return (int) spinner.getValue();
	}

	/*public int getMinimum() {
		return (int) model.getMinimum();
	}

	public int getMaximum() {
		return (int) model.getMaximum();
	}*/

	public void setIncrement(int i) {
		inc = i;
		model.setStepSize(inc);
	}

	public synchronized void setValue(int value) {
		spinner.setValue(value);
	}

	public void setMinimum(int min) {
		model.setMinimum(min);
	}

	public void setMaximum(int max) {
		model.setMaximum(max);
	}
}
